import { autocompletion } from "@codemirror/autocomplete";
import { EditorView, type Tooltip, hoverTooltip } from "@codemirror/view";
import { CompletionTriggerKind } from "vscode-languageserver-protocol";
import type { CompletionContext, CompletionResult } from "@codemirror/autocomplete";
import type { Extension } from "@codemirror/state";
import type { PluginValue, ViewUpdate } from "@codemirror/view";
import type { Transport } from "@open-rpc/client-js/build/transports/Transport.js";
import type { PublishDiagnosticsParams } from "vscode-languageserver-protocol";
import type * as LSP from "vscode-languageserver-protocol";
interface LSPRequestMap {
    initialize: [LSP.InitializeParams, LSP.InitializeResult];
    "textDocument/hover": [LSP.HoverParams, LSP.Hover];
    "textDocument/completion": [
        LSP.CompletionParams,
        LSP.CompletionItem[] | LSP.CompletionList | null
    ];
    "completionItem/resolve": [LSP.CompletionItem, LSP.CompletionItem];
    "textDocument/definition": [
        LSP.DefinitionParams,
        LSP.Definition | LSP.DefinitionLink[] | null
    ];
    "textDocument/codeAction": [
        LSP.CodeActionParams,
        (LSP.Command | LSP.CodeAction)[] | null
    ];
    "textDocument/rename": [LSP.RenameParams, LSP.WorkspaceEdit | null];
    "textDocument/prepareRename": [
        LSP.PrepareRenameParams,
        LSP.Range | LSP.PrepareRenameResult | null
    ];
    "textDocument/signatureHelp": [
        LSP.SignatureHelpParams,
        LSP.SignatureHelp | null
    ];
}
interface LSPNotifyMap {
    initialized: LSP.InitializedParams;
    "textDocument/didChange": LSP.DidChangeTextDocumentParams;
    "textDocument/didOpen": LSP.DidOpenTextDocumentParams;
}
interface LSPEventMap {
    "textDocument/publishDiagnostics": LSP.PublishDiagnosticsParams;
}
type Notification = {
    [key in keyof LSPEventMap]: {
        jsonrpc: "2.0";
        id?: null | undefined;
        method: key;
        params: LSPEventMap[key];
    };
}[keyof LSPEventMap];
export declare class LanguageServerClient {
    ready: boolean;
    capabilities: LSP.ServerCapabilities | null;
    initializePromise: Promise<void>;
    private rootUri;
    private workspaceFolders;
    private autoClose?;
    private timeout;
    private transport;
    private requestManager;
    private client;
    private initializationOptions;
    clientCapabilities: LanguageServerClientOptions["capabilities"];
    private plugins;
    constructor({ rootUri, workspaceFolders, transport, autoClose, initializationOptions, capabilities, timeout, }: LanguageServerClientOptions);
    protected getInitializationOptions(): LSP.InitializeParams["initializationOptions"];
    initialize(): Promise<void>;
    close(): void;
    textDocumentDidOpen(params: LSP.DidOpenTextDocumentParams): Promise<LSP.DidOpenTextDocumentParams>;
    textDocumentDidChange(params: LSP.DidChangeTextDocumentParams): Promise<LSP.DidChangeTextDocumentParams>;
    textDocumentHover(params: LSP.HoverParams): Promise<LSP.Hover>;
    textDocumentCompletion(params: LSP.CompletionParams): Promise<LSP.CompletionItem[] | LSP.CompletionList | null>;
    completionItemResolve(item: LSP.CompletionItem): Promise<LSP.CompletionItem>;
    textDocumentDefinition(params: LSP.DefinitionParams): Promise<LSP.Definition | LSP.LocationLink[] | null>;
    textDocumentCodeAction(params: LSP.CodeActionParams): Promise<(LSP.Command | LSP.CodeAction)[] | null>;
    textDocumentRename(params: LSP.RenameParams): Promise<LSP.WorkspaceEdit | null>;
    textDocumentPrepareRename(params: LSP.PrepareRenameParams): Promise<LSP.PrepareRenameResult | null>;
    textDocumentSignatureHelp(params: LSP.SignatureHelpParams): Promise<LSP.SignatureHelp | null>;
    attachPlugin(plugin: LanguageServerPlugin): void;
    detachPlugin(plugin: LanguageServerPlugin): void;
    protected request<K extends keyof LSPRequestMap>(method: K, params: LSPRequestMap[K][0], timeout: number): Promise<LSPRequestMap[K][1]>;
    protected notify<K extends keyof LSPNotifyMap>(method: K, params: LSPNotifyMap[K]): Promise<LSPNotifyMap[K]>;
    protected processNotification(notification: Notification): void;
}
export declare class LanguageServerPlugin implements PluginValue {
    private documentVersion;
    client: LanguageServerClient;
    documentUri: string;
    languageId: string;
    view: EditorView;
    allowHTMLContent: boolean;
    sendIncrementalChanges: boolean;
    featureOptions: Required<FeatureOptions>;
    onGoToDefinition: ((result: DefinitionResult) => void) | undefined;
    constructor(client: LanguageServerClient, documentUri: string, languageId: string, view: EditorView, featureOptions: Required<FeatureOptions>, sendIncrementalChanges?: boolean, allowHTMLContent?: boolean, onGoToDefinition?: (result: DefinitionResult) => void);
    update({ state, docChanged, startState: { doc }, changes, }: ViewUpdate): void;
    destroy(): void;
    initialize({ documentText }: {
        documentText: string;
    }): Promise<void>;
    sendChanges(contentChanges: LSP.TextDocumentContentChangeEvent[]): Promise<void>;
    requestDiagnostics(view: EditorView): void;
    requestHoverTooltip(view: EditorView, { line, character }: {
        line: number;
        character: number;
    }): Promise<Tooltip | null>;
    requestCompletion(context: CompletionContext, { line, character }: {
        line: number;
        character: number;
    }, { triggerKind, triggerCharacter, }: {
        triggerKind: CompletionTriggerKind;
        triggerCharacter: string | undefined;
    }): Promise<CompletionResult | null>;
    requestDefinition(view: EditorView, { line, character }: {
        line: number;
        character: number;
    }): Promise<DefinitionResult | undefined>;
    processNotification(notification: Notification): void;
    processDiagnostics(params: PublishDiagnosticsParams): Promise<void>;
    private requestCodeActions;
    requestRename(view: EditorView, { line, character }: {
        line: number;
        character: number;
    }): Promise<void>;
    /**
     * Request signature help from the language server
     * @param view The editor view
     * @param position The cursor position
     * @returns A tooltip with the signature help information or null if not available
     */
    requestSignatureHelp(view: EditorView, { line, character, }: {
        line: number;
        character: number;
    }, triggerCharacter?: string | undefined): Promise<Tooltip | null>;
    /**
     * Shows a signature help tooltip at the specified position
     */
    showSignatureHelpTooltip(view: EditorView, pos: number, triggerCharacter?: string): Promise<void>;
    /**
     * Creates the main tooltip container for signature help
     */
    private createTooltipContainer;
    /**
     * Creates the signature element with parameter highlighting
     */
    private createSignatureElement;
    /**
     * Applies parameter highlighting using a range approach
     */
    private applyRangeHighlighting;
    /**
     * Creates the documentation element for signatures
     */
    private createDocumentationElement;
    /**
     * Creates the parameter documentation element
     */
    private createParameterDocElement;
    /**
     * Fallback implementation of prepareRename.
     * We try to find the word at the cursor position and return the range of the word.
     */
    private prepareRenameFallback;
    /**
     * Apply workspace edit from rename operation
     * @param view The editor view
     * @param edit The workspace edit to apply
     * @returns True if changes were applied successfully
     */
    protected applyRenameEdit(view: EditorView, edit: LSP.WorkspaceEdit | null): Promise<boolean>;
}
/**
 * Options for configuring the language server client
 */
interface LanguageServerClientOptions {
    /** The root URI of the workspace, used for LSP initialization */
    rootUri: string;
    /** List of workspace folders to send to the language server */
    workspaceFolders: LSP.WorkspaceFolder[] | null;
    /** Transport mechanism for communicating with the language server */
    transport: Transport;
    /** Whether to automatically close the connection when the editor is destroyed */
    autoClose?: boolean;
    /** Timeout for requests to the language server */
    timeout?: number;
    /**
     * Client capabilities to send to the server during initialization.
     * Can be an object or a function that modifies the default capabilities.
     */
    capabilities?: LSP.InitializeParams["capabilities"] | ((defaultCapabilities: LSP.InitializeParams["capabilities"]) => LSP.InitializeParams["capabilities"]);
    /** Additional initialization options to send to the language server */
    initializationOptions?: LSP.InitializeParams["initializationOptions"];
}
/**
 * Keyboard shortcut configuration for LSP features
 */
interface KeyboardShortcuts {
    /** Keyboard shortcut for rename operations (default: F2) */
    rename?: string;
    /** Keyboard shortcut for go to definition (default: Ctrl/Cmd+Click) */
    goToDefinition?: string;
    /** Keyboard shortcut for signature help (default: Ctrl/Cmd+Shift+Space) */
    signatureHelp?: string;
}
/**
 * Result of a definition lookup operation
 */
interface DefinitionResult {
    /** URI of the target document containing the definition */
    uri: string;
    /** Range in the document where the definition is located */
    range: LSP.Range;
    /** Whether the definition is in a different file than the current document */
    isExternalDocument: boolean;
}
export interface FeatureOptions {
    /** Whether to enable diagnostic messages (default: true) */
    diagnosticsEnabled?: boolean;
    /** Whether to enable hover tooltips (default: true) */
    hoverEnabled?: boolean;
    /** Whether to enable code completion (default: true) */
    completionEnabled?: boolean;
    /** Whether to enable go-to-definition (default: true) */
    definitionEnabled?: boolean;
    /** Whether to enable rename functionality (default: true) */
    renameEnabled?: boolean;
    /** Whether to enable code actions (default: true) */
    codeActionsEnabled?: boolean;
    /** Whether to enable signature help (default: true) */
    signatureHelpEnabled?: boolean;
    /** Whether to show signature help while typing (default: false) */
    signatureActivateOnTyping?: boolean;
}
/**
 * Complete options for configuring the language server integration
 */
interface LanguageServerOptions extends FeatureOptions {
    /** Pre-configured language server client instance or options */
    client: LanguageServerClient;
    /** Whether to allow HTML content in hover tooltips and other UI elements */
    allowHTMLContent?: boolean;
    /** URI of the current document being edited. If not provided, must be passed via the documentUri facet. */
    documentUri?: string;
    /** Language identifier (e.g., 'typescript', 'javascript', etc.). If not provided, must be passed via the languageId facet. */
    languageId?: string;
    /** Configuration for keyboard shortcuts */
    keyboardShortcuts?: KeyboardShortcuts;
    /** Callback triggered when a go-to-definition action is performed */
    onGoToDefinition?: (result: DefinitionResult) => void;
    /**
     * Configuration for the completion feature.
     * If not provided, the default completion config will be used.
     */
    completionConfig?: Parameters<typeof autocompletion>[0];
    /**
     * Configuration for the hover feature.
     * If not provided, the default hover config will be used.
     */
    hoverConfig?: Parameters<typeof hoverTooltip>[1];
    /**
     * Regular expression for determining when to show completions.
     * Default is to show completions when typing a word, after a dot, or after a slash.
     */
    completionMatchBefore?: RegExp;
    /**
     * Whether to send incremental changes to the language server.
     * @default true
     */
    sendIncrementalChanges?: boolean;
}
/**
 * Options for connecting to a language server via WebSocket
 */
interface LanguageServerWebsocketOptions extends Omit<LanguageServerOptions, "client">, Omit<LanguageServerClientOptions, "transport"> {
    /** WebSocket URI for connecting to the language server */
    serverUri: `ws://${string}` | `wss://${string}`;
}
export declare function languageServer(options: LanguageServerWebsocketOptions): Extension[];
export declare function languageServerWithClient(options: LanguageServerOptions): Extension[];
export declare function getCompletionTriggerKind(context: CompletionContext, triggerCharacters: string[], matchBeforePattern?: RegExp): {
    triggerKind: 1 | 2;
    triggerCharacter: string | undefined;
} | null;
export {};
//# sourceMappingURL=plugin.d.ts.map